@@ -3,7 +3,7 @@
 """Non-graphical part of the DFTB+ step in a SEAMM flowchart
 """
 
-import collections
+import collections.abc
 
 try:
     import importlib.metadata as implib
@@ -54,7 +54,7 @@ def deep_merge(d, u):
     while stack:
         d, u = stack.pop(0)
         for k, v in u.items():
-            if not isinstance(v, collections.Mapping):
+            if not isinstance(v, collections.abc.Mapping):
                 # u[k] is not a dict, nothing to merge, so just set it,
                 # regardless if d[k] *was* a dict
                 d[k] = v
@@ -65,7 +65,7 @@ def deep_merge(d, u):
                 # exist
                 dv = d.setdefault(k, {})
 
-                if not isinstance(dv, collections.Mapping):
+                if not isinstance(dv, collections.abc.Mapping):
                     # d[k] is not a dict, so just set it to u[k],
                     # overriding whatever it was
                     d[k] = v
